import { PlayerMoveSide } from "../../constants";
import { printLine, wait } from "../../../common/utilities";
import { AbstractPlayer } from "../../entities/player/AbstractPlayer";
import { Board } from "../Board";
import { PlayerMove } from "../PlayerMove";
import { SimulatedBoard } from "../../SimulatedBoard";
import { Tile } from "../Tile";
import { PRNG } from "seedrandom";
import { PlayerInteractionInterface } from "../../PlayerInteractionInterface";
import { PlayerInteractionAI } from "../../PlayerInteractionAI";
import { InteractionService } from "../../../server/services/InteractionService";

export class PlayerAI extends AbstractPlayer {
  playerInteraction: PlayerInteractionInterface = new PlayerInteractionAI(this);

  constructor(name: string, public rng: PRNG, public sessionId: string) {
    super(name);
  } 

  chooseTileRandom(board: Board): Tile | null { // random algorithm
    const validTiles: Tile[] = this.hand.filter(tile => board.isValidMove(tile, null, this));
    return validTiles[Math.floor(this.rng() * validTiles.length)];
  }

  chooseTileMinMax(board: Board, depth: number = 3): Tile | null { // minmax algorithm
    const bestMove: Tile | null = null;
    let bestScore: number = -Infinity;
    const validMoves: PlayerMove[] = board.getValidMoves(this);

    validMoves.forEach(move => {
      const simulatedBoard = new SimulatedBoard([ ...board.tiles ], this.rng);
      simulatedBoard.play(move);
      const score = this.minmax(simulatedBoard, depth - 1, false);
    });

    return null;
  }

  minmax(simulatedBoard: SimulatedBoard, depth: number, isMaximizing: boolean): number {
    if (depth === 0 || simulatedBoard.isGameOver) {
      return simulatedBoard.evaluate();
    }

    if (isMaximizing) {
      let maxEval = -Infinity;
      const validMoves = simulatedBoard.getValidMoves(this);
      validMoves.forEach((move: PlayerMove) => {
        const newSimulatedBoard = new SimulatedBoard([ ...simulatedBoard.tiles ], this.rng);
        newSimulatedBoard.play(move);
        const evaluation: number = this.minmax(newSimulatedBoard, depth - 1, false);
        maxEval = Math.max(maxEval, evaluation);
      });
      return maxEval;
    }
     else {
      let minEval = Infinity;
      const validMoves = simulatedBoard.getValidMoves(this);
      validMoves.forEach((move: PlayerMove) => {
        const newSimulatedBoard = new SimulatedBoard([ ...simulatedBoard.tiles ], this.rng);
        newSimulatedBoard.play(move);
        const evaluation: number = this.minmax(newSimulatedBoard, depth - 1, true);
        minEval = Math.min(minEval, evaluation);
      });
      return minEval;
     }

    // if (isMaximizing) {
    //   let bestScore = -Infinity;
    //   boardTiles.forEach(tile => {
    //     const newBoardTiles = [ ...boardTiles ];
    //     newBoardTiles.push(tile);
    //     const score = this.minmax(newBoardTiles, depth - 1, false);
    //     bestScore = Math.max(score, bestScore);
    //   });
    //   return bestScore;
    // } else {
    //   let bestScore = Infinity;
    //   boardTiles.forEach(tile => {
    //     const newBoardTiles = [ ...boardTiles ];
    //     newBoardTiles.push(tile);
    //     const score = this.minmax(newBoardTiles, depth - 1, true);
    //     bestScore = Math.min(score, bestScore);
    //   });
      // return bestScore;
    // }
  }

  evaluateBoard(board: Board): number {
    // Custom heuristic to evaluate the board state
    return board.tiles.length; // Simplistic example
  }
}