import { Request, Response } from "express";
import { ApiTokenMongoManager } from "../db/mongo/ApiTokenMongoManager";
import { SecurityManager } from "../managers/SecurityManager";
import { BaseController } from "./BaseController";
import { Token } from "../db/interfaces";
import toObjectId from "../db/mongo/common/mongoUtils";

export class ApiKeyController extends BaseController{
  apiTokenManager = new ApiTokenMongoManager();
  security = new SecurityManager();

  async deleteApiKey(req: Request, res: Response) {
    try {
      const { id } = req.params;
      await this.apiTokenManager.deleteToken(id);
      res.status(200).end();
    } catch (error) {
      this.handleError(res, error);
    }
  }

  async createApiKey(req: Request, res: Response) {
    try {
      const token: Token = this._createTokenObject(req);
      await this.apiTokenManager.create(token);
      res.status(201).end();
    } catch (error) {
      this.handleError(res, error);
    }
  }

  async listUserApiKeys(req: Request, res: Response) {
    try {
      const { user } = req;
      const response = await this.apiTokenManager.getTokens(user._id);
      res.json(response).status(200).end();
    } catch (error) {
      this.handleError(res, error);
    }
  }

  async listNamespaceApiKeys(req: Request, res: Response) {
    try {
      const { namespaceId } = req.user;
      const response = await this.apiTokenManager.getTokensByNamespace(namespaceId);
      res.json(response).status(200).end();
    } catch (error) {
      this.handleError(res, error);
    }
  }

  async deleteNamespaceApiKey(req: Request, res: Response) {
    try {
      const { tokenId: id } = req.params;
      await this.apiTokenManager.deleteToken(id);
      res.status(200).end();
    } catch (error) {
      this.handleError(res, error);
    }
  }

  async createNamespaceApiKey(req: Request, res: Response) {
    try {
      const token = this._createTokenObject(req);
      await this.apiTokenManager.create(token);
      res.status(201).end();
    } catch (error) {
      this.handleError(res, error);
    }
  }

  _createTokenObject(req: Request): Token {
    const { user, body } = req;
    const { roles = [ 'client' ], type = 'user', description = '' } = body;
    const { _id: userId, namespaceId } = user;
    const token = this.security.generateApiToken();
    const newToken: Token =  { 
      token, 
      description, 
      roles, 
      type 
    };
    if (type === 'namespace') {
      newToken.namespaceId = namespaceId;
    } else if (type === 'user') {
      newToken.userId = userId;
    }
    return newToken;
  }
}