import { Request, Response } from "express";
import { BaseController } from "./BaseController";
import { SessionService } from "../services/SessionService";

export class GameController extends BaseController {
  private sessionService: SessionService = new SessionService();

  public async createMatch(req: Request, res: Response) {
    try {
      const { user, body } = req;
      const { sessionName, seed } = body;
      const sessionId = await this.sessionService.createSession(user, sessionName, seed);
      res.status(201).json({ sessionId });
    } catch (error) {
      this.handleError(res, error);
    }
  }

  public async joinMatch(req: Request, res: Response) {
    try {
      const { user, params } = req;
      const { sessionId } = params;
      await this.sessionService.joinSession(user, sessionId);
      res.status(200).json({ status: 'ok' });
    } catch (error) {
      this.handleError(res, error);
    }
  }

  public async listMatches(req: Request, res: Response) {
    try {
      const sessions = await this.sessionService.listJoinableSessions()
      res.status(200).json(sessions);
    } catch (error) {
      this.handleError(res, error);
    }
  }

  public async getMatch(req: Request, res: Response) {
    try {
      const { sessionId } = req.params;
      const session = this.sessionService.getSession(sessionId)
      res.status(200).json(session);
    } catch (error) {
      this.handleError(res, error);
    }
  }

  public async deleteMatch(req: Request, res: Response) {
    try {
      const { sessionId } = req.params;
      await this.sessionService.deleteSession(sessionId);
      this.logger.info(`Session ${sessionId} deleted`);
      res.status(200).json({ status: 'ok' });
    } catch (error) {
      this.handleError(res, error);
    }
  }
}