import { mongoExecute } from './common/mongoDBPool';
import { BaseMongoManager } from './common/BaseMongoManager';
import { Token } from '../interfaces';

export class ApiTokenMongoManager  extends BaseMongoManager{
  collection = 'tokens';

  async getTokens(userId: string): Promise<Token[]> {
    return await mongoExecute(async ({ collection }) => {
      return await collection?.find({ userId: this.toObjectId(userId) }).toArray();
    }, { colName: this.collection });
  }

  async getTokensByNamespace(namespaceId: string): Promise<Token[]> {
    return await mongoExecute(async ({ collection }) => {
      return await collection?.find({ namespaceId: this.toObjectId(namespaceId) }).toArray();
    }, { colName: this.collection });
  }

  async getByToken(token: string): Promise<Token | null>{
    return await mongoExecute(async ({ collection }) => {
      return await collection?.findOne({ token });
    }, { colName: this.collection });
  }

  async deleteToken(tokenId: string): Promise<number> {
    return await mongoExecute(async ({ collection }) => {
      const res = await collection?.deleteOne({ _id: this.toObjectId(tokenId) });
      return res?.deletedCount || 0
    }, { colName: this.collection });
  }
}