import { DeleteResult } from "mongodb";
import { ApiTokenMongoManager } from "./ApiTokenMongoManager";
import { mongoExecute } from "./common/mongoDBPool";
import { Token } from "../interfaces";

export class TemporalTokenMongoManager extends ApiTokenMongoManager{
  collection = 'temporalTokens';

  static Types = {
    PASSWORD_RECOVERY: 'password-recovery',
  };

  async getTokens(): Promise<Token[]> {
    return this.getAllTokens();
  }

  async getAllTokens(): Promise<Token[]> {
    return await mongoExecute(async ({ collection }) => {
      return await collection?.find({ }).toArray();
    }, { colName: this.collection });
  }

  async getByUserAndType(userId: string, type: String): Promise<Token | null>{
    return await mongoExecute(async ({ collection }) => {
      return await collection?.findOne({ userId: this.toObjectId(userId) , type });
    }, { colName: this.collection });
  }

  async deleteAllByUserAndType(userId: string, type: String): Promise<number>{
    return await mongoExecute(async ({ collection }) => {
      const res: DeleteResult | undefined = await collection?.deleteMany({ userId: this.toObjectId(userId), type });
      return res?.deletedCount || 0;
    }, { colName: this.collection });
  }
}