import { MongoClient, Collection, Db } from 'mongodb';

const {
  MONGO_HOST,
  MONGO_PORT,
  MONGO_USER,
  MONGO_PASS = '',
  MONGO_DB,
} = process.env;

const uri = `mongodb://${MONGO_USER}:${MONGO_PASS.replace(/[^A-Za-z0-9\-_.!~*'()%]/g, (c) => encodeURIComponent(c))}@${MONGO_HOST}:${MONGO_PORT}/?maxPoolSize=20`;

interface MongoExecuteOptions {
  dbName?: string;
  colName?: string;
};

interface MongoExecuteParams {
  collection?: Collection, 
  database?: Db, 
  connection?: MongoClient
}

type MongoExecuteFunction = (options: MongoExecuteParams) => void | Promise<void> | Promise<any> | any;

export const getMongoConnection = async() : Promise<MongoClient> => {
  const client = new MongoClient(uri);
  return await client.connect();
};

export const getMongoDatabase = (client: MongoClient, dbName: string): Db => {
  const DB = dbName || MONGO_DB;
  return client.db(DB);
};

export const mongoExecute = async function(fn: MongoExecuteFunction, opts: MongoExecuteOptions): Promise<any> {
  const { dbName, colName } = { dbName: MONGO_DB, ...opts };
  let connection: MongoClient | null = null;
  try {
    connection = await getMongoConnection();
    const database = connection.db(dbName);
    if (colName) {
      const collection: Collection = database.collection(colName);
      return await fn({ collection, database, connection });
    }
    return await fn({ database, connection });
  } catch (err: any) {
    console.log('MOMGODB ERROR:', err.message);
    throw err;
  } finally {
    if (connection !== null) {
      await connection.close();
    }
  }
};
