import { Request, Response, Router } from 'express';
import { AuthController } from '../controllers/AuthController';

import adminRouter from './adminRouter';
import userRouter from './userRouter';
import gameRouter from './gameRouter';

export default function(): Router {
  const router = Router();
  const authController = new AuthController();

  router.get('/version', async function(req: Request, res: Response){
      res.send('1.0.0').end();
  });

  router.post('/auth/code', (req: Request, res: Response) => authController.twoFactorCodeAuthentication(req, res));
  router.post('/login', (req: Request, res: Response) => authController.login(req, res));

  router.use('/admin', adminRouter());
  router.use('/user', userRouter());
  router.use('/game', gameRouter());


  return router;
}
