import { Request, Response, Router } from 'express';
import { UserController } from '../controllers/UserController';
import { AuthController } from '../controllers/AuthController';
import { ApiKeyController } from '../controllers/ApiKeyController';

export default  function() : Router {
  const userController = new UserController();
  const authController = new AuthController();
  const apiKeyController = new ApiKeyController();
  const router = Router();

  const { authenticate } = AuthController;

  router.get('/tokens', authenticate({ roles: ['user']}), (req: Request, res: Response) => apiKeyController.listUserApiKeys(req, res));
  router.post('/token', authenticate({ roles: ['user']}), (req: Request, res: Response) => apiKeyController.createApiKey(req, res));
  router.delete('/token/:id', authenticate({ roles: ['user']}), (req: Request, res: Response) => apiKeyController.deleteApiKey(req, res));
  router.post('/password/change', authenticate({ roles: ['user']}), (req: Request, res: Response) => authController.changePassword(req, res));
  router.post('/password/recovery', (req: Request, res: Response) => userController.passwordRecovery(req, res));
  router.post('/password/recovery/change', (req: Request, res: Response) => authController.changePasswordWithCode(req, res));
  router.get('/namespaces', authenticate({ roles: ['user']}), (req: Request, res: Response) => userController.getNamespaces(req, res));

  return router;
}